//
//  ViewController.swift
//  PresenceSDK-DemoIntegration
//
//  Created by Varun Bansal on 2/9/18.
//  Copyright © 2018 Ticketmaster. All rights reserved.
//

import PresenceSDK

class PresenceViewController: UIViewController {
    
    let presenceSDK: PresenceSDK = PresenceSDK.getPresenceSDK()
    
    //Connect your PresenceSDKView here
    @IBOutlet weak var presenceSDKView: PresenceSDKView? = nil

    override func viewDidLoad() {
        super.viewDidLoad()
        
        addSDKNotificationHandlers()
        
        presenceSDK.start(presenceSDKView: presenceSDKView, loginDelegate: self)
    }
}

// MARK: - PresenceLoginDelegate
extension PresenceViewController: PresenceLoginDelegate {
    
    // All methods are optional

    /**
     Method is invoked if the user granted app access/logged in.
     - parameter backendName: Name of the backend this callback event is associated with.
     - parameter accessToken: access token returned from the specified backend.
     */
    func onLoginSuccessful(backendName: PresenceLogin.BackendName, accessToken: String) {
        print("Login was successful for backend: \(backendName) and the access token is: \(accessToken)")
    }
    
    /**
    User dismissed login window via the Cancel button
    - parameter backendName: Name of the backend this callback event is associated with.
    */
    func onLoginCancelled(backendName: PresenceLogin.BackendName) {
        print("Login was cancelled for backend: \(backendName)")
    }
    
    /**
     Called when results are returned for a member info request after successful login
     - parameter backendName: Name of the backend this callback event is associated with.
     - parameter member: PresenceMember object. PresenceMember object is `nil` if login fails or an error is returned fetching member details.
     */
    func onMemberUpdated(backendName: PresenceLogin.BackendName, member: PresenceMember?) {
        printMemberInfo(member: member)
    }
    
    /**
     - parameter backendName: Name of the backend this callback event is associated with.
     - parameter error: If available, an `NSError` object is returned. Defaults is `nil`.
     */
    func onLoginFailed(backendName: PresenceLogin.BackendName, error: NSError?) {
        if let loginError = error {
            print("Login failed for backend: \(backendName) because of error: \(loginError.localizedDescription)");
        }
    }
    
    /**
     Notify when successfully logged-out from both backends
     */
    func onLogoutAllSuccessful() {
        print("Logout successful")
    }
    
    /**
     Notify when user clicks on "forgot password" link
     */
    func onLoginForgotPasswordClicked() {
        
    }
    
    /**
     Notify when all cache is cleared
     */
    func onCacheCleared() {
        
    }
    
    /**
     Notify when successfully logged-out
     - parameter backendName: Name of the backend this callback event is associated with.
     */
    func onLogoutSuccessful(backendName: PresenceLogin.BackendName) {
        
    }
    
    /**
     Called when the LoginWindow is made visible to the user.
     */
    func loginWindowDidDisplay() {
        
    }
}


//MARK: IBAction Methods
fileprivate extension PresenceViewController {
    
    @IBAction func logout(sender: UIButton) {
        presenceSDK.logOut()
    }
}

//MARK: Private Helper Methods
fileprivate extension PresenceViewController {
    
    /**
     Method to print member information
     */
    func printMemberInfo(member: PresenceMember?) {
        guard let presenceMember = member else {
            return
        }
        
        print("Member Email: \(presenceMember.emailAddress)")
        print("Team Member-Id: \(presenceMember.AccountManagerMemberID)")
        print("Host Member-Id: \(presenceMember.HostMemberID)")
    }
}

//MARK: Presence SDK Local Notification Handlers
fileprivate extension PresenceViewController {
    
    func addSDKNotificationHandlers() {
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(self.ticketsScreenShown),
                                               name: NSNotification.Name(rawValue: PresenceEventAnalytics.Action.ACTION_MANAGETICKETSCREENSHOWED),
                                               object: nil)
    }
    
    /**
     Called when user selects a particular event to view tickets.
     */
    @objc func ticketsScreenShown(_ notification: Notification) {
        guard let userInfo = notification.userInfo else {
            return
        }
        
        let eventId = userInfo[PresenceEventAnalytics.Data.EVENT_ID] ?? ""
        let eventName = userInfo[PresenceEventAnalytics.Data.EVENT_NAME] ?? ""
        print ("Event_Id: \(eventId), Event_Name: \(eventName)")
    }
}
